#include "07_monitor.h"
#include <stdlib.h>
#include <stdio.h>

/*
 * Approximation of monitor semantics
 * Simplified for educational reasons
 */

Monitor *createMonitor()
{
	Monitor *result = (Monitor *)malloc(sizeof(Monitor));
	if(!result)
		return result;
	pthread_mutex_init(&result->lock, NULL);
	pthread_mutex_init(&result->notifier, NULL);

	return result;
}

void destroyMonitor(Monitor *mon)
{
	pthread_mutex_destroy(&mon->lock);
	pthread_mutex_destroy(&mon->notifier);
}

void monitorEnter(Monitor *mon)
{
	pthread_mutex_lock(&mon->lock);
}

void monitorExit(Monitor *mon)
{
	pthread_mutex_unlock(&mon->lock);
}

/*
 * May only be called between enter and exit!
 */
void monitorWaitOn(Monitor *mon)
{
	/* returns 0 on success! */
	/* printf("Entering wait\n");*/
	int fail = pthread_mutex_trylock(&mon->notifier);
	if(fail)
        {
	/*	printf("Starting to wait\n"); */
		pthread_mutex_unlock(&mon->lock);

	/*	printf("Wait locking notifier\n"); */
		pthread_mutex_lock(&mon->notifier);
		
		/*
		 * Cannot continue until other thread has finished!
		 */
	/*	printf("Wait locking lock\n"); */
		pthread_mutex_lock(&mon->lock);
	}
	/*printf("Wait continuing\n"); */
}

/*
 * May only be called between enter and exit!
 */
void monitorNotifyWaiting(Monitor *mon)
{
	/*printf("Notifying\n");*/
	pthread_mutex_unlock(&mon->notifier);
}
